!INTERFILE  :=
name of data file := scatter_cylinder.s
originating system := unknown
!GENERAL DATA :=
!GENERAL IMAGE DATA :=
!type of data := PET
imagedata byte order := LITTLEENDIAN
!PET STUDY (General) :=
!PET data type := Emission
applied corrections := {None}
!number format := float
!number of bytes per pixel := 4
number of dimensions := 4
matrix axis label [4] := segment
!matrix size [4] := 1
matrix axis label [3] := view
!matrix size [3] := 32
matrix axis label [2] := axial coordinate
!matrix size [2] := { 8}
matrix axis label [1] := tangential coordinate
!matrix size [1] := 35
minimum ring difference per segment := { 0}
maximum ring difference per segment := { 0}
Scanner parameters:= 
Scanner type := unknown
Number of rings                          := 8
Number of detectors per ring             := 64
Inner ring diameter (cm)                 := 88.62
Average depth of interaction (cm)        := 0.84
Distance between rings (cm)              := 1.962
Default bin size (cm)                    := -1
View offset (degrees)                    := 0
Maximum number of non-arc-corrected bins := 0
Default number of arc-corrected bins     := 0
Number of blocks per bucket in transaxial direction         := 0
Number of blocks per bucket in axial direction              := 0
Number of crystals per block in axial direction             := 0
Number of crystals per block in transaxial direction        := 0
Number of detector layers                                   := 1
Number of crystals per singles unit in axial direction      := -1
Number of crystals per singles unit in transaxial direction := -1
end scanner parameters:=
effective central bin size (cm) := 2.21541
image scaling factor[1] := 1
data offset in bytes[1] := 0
number of time frames := 1
!END OF INTERFILE :=
