//
 /*
  Copyright (C) 2009 -2013, King's College London
  This file is part of STIR.
  
  This file is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.3 of the License, or
  (at your option) any later version.
  
  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
  
  See STIR/LICENSE.txt for details
*/  
/*!
  \file
  \ingroup spatial_transformation
  \brief File that registers all stir::RegisterObject children in spatial_transformation

  \author Charalampos Tsoumpas
  
*/

#include "stir/spatial_transformation/GatedSpatialTransformation.h"

START_NAMESPACE_STIR

static GatedSpatialTransformation::RegisterIt dummy1199;

END_NAMESPACE_STIR

