//
//
/*
    Copyright (C) 2000- 2005, Hammersmith Imanet Ltd
    For internal GE use only
*/

// define some doxygen groups and namespace
// This file does not contain any code
/*!
\defgroup motion Items relating to motion and motion correction
\ingroup STIR_library
*/

/*!
\defgroup motion_utilities Utility programs for motion and motion correction
\ingroup utilities
*/
/*!
\defgroup motion_test Tests of motion transformations etc
\ingroup alltest
*/
