#! /bin/sh
# SUPERSEDED SCRIPT. FUNCTIONALITY IN STIR
# Preliminary script to convert Interfile headers from Siemens format to STIR
# WARNING: this currently doesn't check a load of things
# for instance, for images, x-y offset and orientation are not checked
# see notes below for sinograms
#
# Author: Kris Thielemans
# Copyright: University College London

# check number of arguments
echo "STIR is currently able to parse Siemens Interfile format. This script ends now."
exit 1
