//
//
// This file is for doxygen only. It does not contain any code.
/*
    Copyright (C) 2000- 2009, Hammersmith Imanet Ltd
    This file is part of STIR.

    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    See STIR/LICENSE.txt for details
*/

/*!
 \mainpage

 \section intro Introduction

 This is the detailed documentation for STIR: Software for Tomographic
 Image Reconstruction. Additional documentation 
 (sometimes overlapping) is given in the User's Guide, 
 Overview documents etc are available on the 
 STIR website (http://stir.sourceforge.net). You should probably read those
 first before looking into the documentation on this page.

 The current pages are automatically
 generated by doxygen from text comments in the files.

 As the amount of software is rather large, some guidance is  
 needed. The best place to start is probably the Modules section. This
 attempts to group files in a sensible way. In particular, a User might only be
 interested in the Executables module. (Note however that the doxygen
 documentation for some executables is not very informative. In any case,
 just executing the utility should give a usage message).

 Another way to navigate through this is the Class Hierarchy page, or indeed
 any of the other alphabetical lists. 
 However, this is not structured at all, and even contains some classes
 which we'd rather not have in there. So, in the beginning, this might be 
 somewhat confusing.

 \section docproblems Problems to keep in mind when reading this documentation

 All documentation depends on the person who writes it. In the end, only 
 the code is what matters...

 

*/
