



cd recon_test_pack
IF %ERRORLEVEL% NEQ 0 exit /B 1
run_tests
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd ..
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest src/swig/test/python
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
