//
//
/*
    Copyright (C) 2000 PARAPET partners
    Copyright (C) 2000- 2005, Hammersmith Imanet Ltd
    This file is part of STIR.

    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    See STIR/LICENSE.txt for details
*/
#ifndef __interpolate_H__
#define __interpolate_H__
/*!
  \file 
  \ingroup buildblock
 
  \brief declares functions for interpolation

  \warning will be moved to numerics
  \author Kris Thielemans
  \author PARAPET project

*/


#include "stir/numerics/overlap_interpolate.h"

#endif
